@@ -39,11 +39,17 @@ extern "C"
 
 #include "liblouis.h"
 
+#ifdef _WIN32
+#define strcasecmp _stricmp
+#endif
+
 #define NUMSWAPS 50
 #define NUMVAR 50
-#define LETSIGNSIZE 16
+#define LETSIGNSIZE 128
 #define CHARSIZE sizeof(widechar)
 #define DEFAULTRULESIZE 50
+#define ENDSEGMENT 0xffff
+
 /*Definitions of braille dots*/
 #define B1 0X01
 #define B2 0X02
@@ -65,7 +71,7 @@ extern "C"
 /*HASHNUM must be prime */
 #define HASHNUM 1123
 
-#define MAXSTRING 256
+#define MAXSTRING 512
 
   typedef unsigned int TranslationTableOffset;
 #define OFFSETSIZE sizeof (TranslationTableOffset)
@@ -89,6 +95,8 @@ extern "C"
 
   typedef enum
   {
+    pass_first = '`',
+    pass_last = '~',
     pass_lookback = '_',
     pass_string = '\"',
     pass_dots = '@',
@@ -124,11 +132,33 @@ extern "C"
     pass_eq = '=',
     pass_lt = '<',
     pass_gt = '>',
-    pass_lteq = 130,
-    pass_gteq = 131,
     pass_endTest = 32,
     pass_plus = '+',
-    pass_copy = '*'
+    pass_copy = '*',
+    pass_leftParen = '(',
+    pass_rightParen = ')',
+    pass_comma = ',',
+    pass_lteq = 130,
+    pass_gteq = 131,
+    pass_invalidToken = 132,
+    pass_noteq = 133,
+    pass_and = 134,
+    pass_or = 135,
+    pass_nameFound = 136,
+    pass_numberFound = 137,
+    pass_boolean = 138,
+    pass_class = 139,
+    pass_define = 140,
+    pass_emphasis = 141,
+    pass_group = 142,
+    pass_mark = 143,
+    pass_repGroup = 143,
+    pass_script = 144,
+    pass_noMoreTokens = 145,
+    pass_replace = 146,
+    pass_if = 147,
+    pass_then = 148,
+    pass_all = 255
   }
   pass_Codes;
 
@@ -159,11 +189,13 @@ extern "C"
   {				/*Op codes */
     CTO_IncludeFile,
     CTO_Locale,			/*Deprecated, do not use */
+    CTO_Undefined,
     CTO_CapitalSign,
     CTO_BeginCapitalSign,
     CTO_LenBegcaps,
     CTO_EndCapitalSign,
     CTO_FirstWordCaps,
+    CTO_LastWordCapsBefore,
     CTO_LastWordCapsAfter,
     CTO_LenCapsPhrase,
     CTO_LetterSign,
@@ -282,6 +314,7 @@ extern "C"
     CTO_BeginCapitalRule,
     CTO_EndCapitalRule,
     CTO_FirstWordCapsRule,
+    CTO_LastWordCapsBeforeRule,
     CTO_LastWordCapsAfterRule,
     CTO_LetterRule,
     CTO_NumberRule,
@@ -365,16 +398,11 @@ extern "C"
     TranslationTableOffset tableSize;
     TranslationTableOffset bytesUsed;
     TranslationTableOffset noBreak;
-    TranslationTableOffset capitalSign;	/*capitalization sign */
-    TranslationTableOffset beginCapitalSign;	/*begin capitals sign */
-    TranslationTableOffset lenBeginCaps;
-    TranslationTableOffset endCapitalSign;	/*end capitals sign */
-    TranslationTableOffset firstWordCaps;
-    TranslationTableOffset lastWordCapsAfter;
-    TranslationTableOffset lenCapsPhrase;
+    TranslationTableOffset undefined;
     TranslationTableOffset letterSign;
-    TranslationTableOffset numberSign;	/*number sign */
-    /*Do not change the order of the following emphasis opcodes */
+    TranslationTableOffset numberSign;
+    /*Do not change the order of the following emphasis rule pointers! 
+     */
     TranslationTableOffset firstWordItal;
     TranslationTableOffset lastWordItalBefore;
     TranslationTableOffset lastWordItalAfter;
@@ -399,6 +427,16 @@ extern "C"
     TranslationTableOffset singleLetterUnder;
     TranslationTableOffset underWord;
     TranslationTableOffset lenUnderPhrase;
+    TranslationTableOffset firstWordCaps;
+    TranslationTableOffset lastWordCapsBefore;
+    TranslationTableOffset lastWordCapsAfter;
+    TranslationTableOffset beginCapitalSign;
+    TranslationTableOffset endCapitalSign;	/*end capitals sign */
+    TranslationTableOffset capitalSign;
+    TranslationTableOffset CapsWord;
+    TranslationTableOffset lenCapsPhrase;
+    /* End of ordered emphasis rule poiinters */
+    TranslationTableOffset lenBeginCaps;
     TranslationTableOffset begComp;
     TranslationTableOffset compBegEmph1;
     TranslationTableOffset compEndEmph1;
@@ -487,6 +525,28 @@ extern "C"
 /* Takes a character string and produces a sequence of wide characters 
 * containing dot patterns. Opposite of showDots. */
 
+  int other_translate (const char *trantab, const widechar
+		       * inbuf,
+		       int *inlen, widechar * outbuf, int *outlen,
+		       char *typeform, char *spacing, int *outputPos, int
+		       *inputPos, int *cursorPos, int mode);
+
+/*Call wrappers for other translators */
+  int other_backTranslate (const char *trantab, const widechar
+			   * inbuf,
+			   int *inlen, widechar * outbuf, int *outlen,
+			   char *typeform, char *spacing, int *outputPos, int
+			   *inputPos, int *cursorPos, int mode);
+/*Call wrappers for other back-translators.*/
+
+
+  int other_dotsToChar (const char *trantab, widechar * inbuf,
+			widechar * outbuf, int length, int mode);
+  int other_charToDots (const char *trantab, const widechar
+			* inbuf, widechar * outbuf, int length, int mode);
+
+  void debugHook ();
+
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
