@@ -3,42 +3,73 @@
    Based on BRLTTY, copyright (C) 1999-2006 by
    The BRLTTY Team
 
-   Copyright (C) 2004, 2005, 2006
-   ViewPlus Technologies, Inc. www.viewplus.com
-   and
+   Copyright (C) 2004, 2005, 2006, 2009
+   ViewPlus Technologies, Inc. www.viewplus.com and
    JJB Software, Inc. www.jjb-software.com
 
-   This file is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; either version 2, or (at your option) any
-   later version.
-
-   In addition to the permissions in the GNU General Public License, the
-   copyright holders give you unlimited permission to link the
-   compiled version of this file into combinations with other programs,
-   and to distribute those combinations without any restriction coming
-   from the use of this file.  (The General Public License restrictions
-   do apply in other respects; for example, they cover modification of
-   the file, and distribution when not linked into a combine
-   executable.)
-
-   This file is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
    You should have received a copy of the GNU General Public License
-   along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
    Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include "louis.h"
+#include <getopt.h>
+#include "progname.h"
+#include "version-etc.h"
+
+static const struct option longopts[] =
+{
+  { "help", no_argument, NULL, 'h' },
+  { "version", no_argument, NULL, 'v' },
+  { NULL, 0, NULL, 0 }
+};
+
+const char version_etc_copyright[] =
+  "Copyright %s %d ViewPlus Technologies, Inc. and JJB Software, Inc.";
+
+#define AUTHORS "John J. Boyer"
+
+static void
+print_help (void)
+{
+  printf ("\
+Usage: %s [OPTION] TABLE\n", program_name);
+  
+  fputs ("\
+Examine and debug Braille translation tables. This program allows you\n\
+to inspect liblouis translation tables and gather information about\n\
+them, such as forward and backward rules, characters and dot patterns,\n\
+specific opcodes, the size of a table, whether a hyphenation\n\
+table is used, how many passes the translation takes and much\n\
+more.\n\n", stdout);
+
+  fputs ("\
+  -h, --help          display this help and exit\n\
+  -v, --version       display version information and exit\n", stdout);
+
+  printf ("\n");
+  printf ("\
+Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+}
+
 #define BUFSIZE 256
 
 static const TranslationTableHeader *table;
@@ -661,17 +692,49 @@ getCommands (void)
 int
 main (int argc, char **argv)
 {
-  if (argc != 2)
+  int optc;
+
+  set_program_name (argv[0]);
+
+  while ((optc = getopt_long (argc, argv, "hv", longopts, NULL)) != -1)
+    switch (optc)
+      {
+      /* --help and --version exit immediately, per GNU coding standards.  */
+      case 'v':
+        version_etc (stdout, program_name, PACKAGE_NAME, VERSION, AUTHORS, (char *) NULL);
+        exit (EXIT_SUCCESS);
+        break;
+      case 'h':
+        print_help ();
+        exit (EXIT_SUCCESS);
+        break;
+      default:
+	fprintf (stderr, "Try `%s --help' for more information.\n",
+		 program_name);
+	exit (EXIT_FAILURE);
+        break;
+      }
+
+  if (optind != argc - 1)
     {
-      fprintf (stderr, "Usage: lou_debug tablename\n");
-      exit (1);
+      /* Print error message and exit.  */
+      if (optind < argc - 1)
+	fprintf (stderr, "%s: extra operand: %s\n",
+		 program_name, argv[optind + 1]);
+      else
+	fprintf (stderr, "%s: no table specified\n", 
+		 program_name);
+      fprintf (stderr, "Try `%s --help' for more information.\n",
+               program_name);
+      exit (EXIT_FAILURE);
     }
-  if (!(table = lou_getTable (argv[1])))
+
+  if (!(table = lou_getTable (argv[optind])))
     {
       lou_free ();
-      return 1;
+      exit (EXIT_FAILURE);
     }
   getCommands ();
   lou_free ();
-  return 0;
+  exit (EXIT_SUCCESS);
 }
